/*
Copyright (c) 2014 DenshiKousakuSenka
 This software is released under the MIT License.
 http://opensource.org/licenses/mit-license.php
 */

#include "DKS_L3GD20.h"
#include <string.h>

void DKS::L3GD20::L3GD20::write(const uint8_t* pBuffer, const uint8_t &WriteAddr, const uint16_t &NumByteToWrite)
{
	const uint8_t MULTIPLEBYTE_CMD = 0x40;
	memset(SPI_Buffer, 0, sizeof(SPI_Buffer));
	SPI_Buffer[0] = WriteAddr;
	if(NumByteToWrite > 1)
	{
		SPI_Buffer[0] |= (uint8_t)MULTIPLEBYTE_CMD;	//7bitڂMulti byte access
	}
	memcpy(&SPI_Buffer[1], pBuffer, NumByteToWrite);
	/* Set chip select Low at the start of the transmission */
	m_ss->write(0);

	m_spi->write(SPI_Buffer, NumByteToWrite + 1 );

	/* Set chip select High at the end of the transmission */
	m_ss->write(1);
}

void DKS::L3GD20::L3GD20::read(uint8_t* pBuffer,const uint8_t &ReadAddr,const uint16_t &NumByteToRead)
{
	const uint8_t READWRITE_CMD = 0x80;
	const uint8_t MULTIPLEBYTE_CMD = 0x40;

	memset(SPI_Buffer, 0, sizeof(SPI_Buffer));
	SPI_Buffer[0] = ReadAddr;
	if(NumByteToRead > 1)
	{
		SPI_Buffer[0] |= (READWRITE_CMD | MULTIPLEBYTE_CMD);
	}
	else
	{
		SPI_Buffer[0] |= READWRITE_CMD;
	}
	/* Set chip select Low at the start of the transmission */
	m_ss->write(0);

	m_spi->write(SPI_Buffer, NumByteToRead+1);

	/* Set chip select High at the end of the transmission */
	m_ss->write(1);
	memcpy(pBuffer, &SPI_Buffer[1], NumByteToRead);

}

DKS::L3GD20::L3GD20::L3GD20(DKS::SPI::SPI_Base *SPI, const DKS::IDigitalOut *SSpin)
	: m_spi(SPI), m_ss(SSpin), m_PowerMode(Active), m_OutputDataRate(DataRate_1),
	  m_BandWidth(BandWidth_4), m_FullScale(FullScale_500)
{
	if (m_spi->getFrequency()>10E6) while (1);	// SPI clock must be less than 10MHz
	m_spi->setFormat(8, DKS::SPI::Mode3);
	m_spi->setBitOrder(DKS::SPI::MSBFirst);

	m_ss->mode(DKS::Push_Pull,DKS::Pull_Up);
	m_ss->write(1);

	///////   mF    ////////////////////
	uint8_t res[1];
	read(res, 0x0F, 1);	//0x0F = "WHO_AM_I"
	if (res[0] != 0xD4) while(1);

	///////   WX^lݒ  ////////////////////
	uint8_t ctrl1 = 0x00, ctrl4 = 0x00;
	ctrl1 |= (uint8_t) (m_PowerMode | m_OutputDataRate | 0x07 | m_BandWidth);	//0x07 = Axes_Enable
	ctrl4 |= (uint8_t) (0x00 | 0x00 | m_FullScale);	//0x00 = Continuous update, Endian MSB First

	write(&ctrl1, 0x20, 1);	//0x20 = CTRL_REG1
	write(&ctrl4, 0x23, 1);	//0x23 = CTRL_REG4

	//Localϐ̐ݒ
	switch(m_FullScale)
	{
	case FullScale_250:
		sensitivity = 0.00875f;
		break;
	case FullScale_500:
		sensitivity = 0.01750f;
		break;
	case FullScale_2000:
		sensitivity = 0.070f;
		break;
	}
}

void DKS::L3GD20::L3GD20::ReadMagData(float Data[3])	//XYZ
{
	uint8_t tmpbuffer[6] ={0};
	int16_t RawData[3] = {0};

	read(tmpbuffer, 0x28, 6);	//0x28 = OUT_X_L

	for(uint8_t i=0; i<3; i++)
	{
		RawData[i]=(int16_t)(((uint16_t)tmpbuffer[2*i+1] << 8) + tmpbuffer[2*i]);	//SCL_L3GD20_BLE_LSBݒ
		Data[i]=(float)RawData[i] * sensitivity;
	}
}

void DKS::L3GD20::L3GD20::ReadMagData(uint16_t Data[3])//XYZ
{
	uint8_t tmpbuffer[6] ={0};

	read(tmpbuffer, 0x28, 6);	//0x28 = OUT_X_L

	for(uint8_t i=0; i<3; i++)
	{
		Data[i]=(int16_t)(((uint16_t)tmpbuffer[2*i+1] << 8) + tmpbuffer[2*i]);	//SCL_L3GD20_BLE_LSBݒ
	}
}

void DKS::L3GD20::L3GD20::ReadTemperature(int8_t &Temperature)
{
	read(reinterpret_cast<uint8_t*>(&Temperature), 0x26, 1);	// 0x26 = OUT_TEMP
}
